/*
    Copyright (C) 2000  Tamas Pal <tpal@egon.gyaloglo.hu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*
  goodnight.c - an XMMS plugin to stop playing at a given time and perform some action
*/   

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <pthread.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <string.h>
#include "xmmsctrl.h"
#include "configfile.h"
#include "plugin.h"

#include "config.h"

#define VERSION "0.1.2"
#define CONFIGFILE "/.xmms/goodnight"

#define STOP "Stop"
#define QUIT "Quit"
#define STANDBY "Standby"
#define SHUTDOWN "Shutdown"

void *goodnight_thread (void *args);
void goodnight_init ();
void goodnight_about ();
void goodnight_config ();
void save_cfgfile (gint xmms_session);
void save_playlist (gint xmms_session);
void
goodnight_config_save (GtkWidget * wid, gpointer data);

static GtkWidget *conf_dialog, *about_win;
static GtkWidget *quit_radio, *shutdown_radio, *standby_radio, *shutdown_entry,
 *standby_entry;
static pthread_t tid;

GtkObject *hour_w, *minute_w, *fadespeed_o;
gint hour = -1, minute = -1, fadespeed = -1;
gchar *shutdown_command = NULL, *standby_command = NULL;
gboolean shutdown = FALSE, quit = FALSE, standby = FALSE;
gchar *dummy="(null)";

void
goodnight_cleanup ()
{
  pthread_cancel (tid);
}

GeneralPlugin goodnight =
{
  NULL,
  NULL,
  -1,
  "XMMS-Goodnight plugin " VERSION,
  goodnight_init,
  goodnight_about,
  goodnight_config,
  goodnight_cleanup,
};

GeneralPlugin *
get_gplugin_info ()
{
  return (&goodnight);
}

void
read_config (GtkWidget * wid, gpointer data)
{
  gchar *configfile, *action=NULL;
  ConfigFile *config;

  configfile = g_strconcat (g_get_home_dir (), CONFIGFILE, NULL);
  if ((config = xmms_cfg_open_file (configfile)) != NULL)
    {
      xmms_cfg_read_int (config, "goodnight", "hour", &hour);
      xmms_cfg_read_int (config, "goodnight", "minute", &minute);
      xmms_cfg_read_int (config, "goodnight", "fadespeed", &fadespeed);
      xmms_cfg_read_string (config, "goodnight", "shutdown_command", &shutdown_command);
      xmms_cfg_read_string (config, "goodnight", "standby_command", &standby_command);

      xmms_cfg_read_string (config, "goodnight", "exit_action", &action);
      xmms_cfg_free (config);
    }
   
  /* some "airbag"... */
  if ((hour < 0) || (hour > 23))
    hour = 0;
  if ((minute < 0) || (minute > 59))
    minute = 0;
  if ((fadespeed < 1) || (fadespeed > 20))
    fadespeed = 2;

  if ( !shutdown_command )
      shutdown_command=dummy;
  if ( !standby_command )
    standby_command=dummy;
  if ( action ){
      if (!strcasecmp (action, QUIT))
        quit = TRUE;
      if (!strcasecmp (action, STANDBY))
        standby = TRUE;
      if (!strcasecmp (action, SHUTDOWN))
        shutdown = TRUE;
   }
  g_free (configfile);
}
void

goodnight_init ()
{
  pthread_attr_t attr;

  read_config (NULL, NULL);

  pthread_attr_init (&attr);
  pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_DETACHED);
  pthread_attr_setinheritsched (&attr, PTHREAD_EXPLICIT_SCHED);
  pthread_attr_setschedpolicy (&attr, SCHED_OTHER);
  pthread_attr_setscope (&attr, PTHREAD_SCOPE_SYSTEM);

  pthread_create (&tid, &attr, goodnight_thread, NULL);
  return;

}

void *
goodnight_thread (void *args)
{
  gint volume;
  struct tm *t_now;
  time_t timenow;
  struct timespec mosec;

  mosec.tv_nsec = 0;

  while (1)
    {
      timenow = time (NULL);
      t_now = localtime (&timenow);

      if (t_now->tm_hour == hour)
	if (t_now->tm_min == minute)
	  {
	    volume = xmms_remote_get_main_volume (goodnight.xmms_session);
	    while (volume >= 10)
	      {
		volume -= fadespeed;
		xmms_remote_set_main_volume (goodnight.xmms_session, volume);
		mosec.tv_sec = 1;
		nanosleep (&mosec, NULL);
	      }
	    xmms_remote_stop (goodnight.xmms_session);
	    save_cfgfile (goodnight.xmms_session);
	    save_playlist (goodnight.xmms_session);
	    if (quit)
#ifdef HAVE_XMMS_REMOTE_QUIT
              xmms_remote_quit(goodnight.xmms_session);
#else
	    /* a bit brute but works...*/
	      raise (SIGTERM);
#endif
	    if ((shutdown) && (shutdown_command))
	      system (shutdown_command);
	    if ((standby) && (standby_command))
	      system (standby_command);
	  }
      mosec.tv_sec = 60;
      nanosleep (&mosec, NULL);
    }
}

void
goodnight_about ()
{
  GtkWidget *button, *label, *bigbox, *buttonbox;

  if (about_win)
    return;
  about_win = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_window_set_title (GTK_WINDOW (about_win), ("About"));
  gtk_window_set_policy (GTK_WINDOW (about_win), FALSE, FALSE, FALSE);
  gtk_window_set_position (GTK_WINDOW (about_win), GTK_WIN_POS_MOUSE);

  bigbox = gtk_vbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER (about_win), bigbox);

  label = gtk_label_new ("XMMS-Goodnight plugin \n\n Stops XMMS at the given time.\n"
  "Created by Tamas Pal <folti@fiktiv.szgtikol.kando.hu>");
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
  gtk_container_add (GTK_CONTAINER (bigbox), label);

  buttonbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (buttonbox), GTK_BUTTONBOX_DEFAULT_STYLE);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (buttonbox), 5);
  gtk_box_pack_start (GTK_BOX (bigbox), buttonbox, FALSE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (about_win), "destroy", GTK_SIGNAL_FUNC (gtk_widget_destroyed),
		      &about_win);

  button = gtk_button_new_with_label ("Ok");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) about_win);
  gtk_box_pack_start (GTK_BOX ((buttonbox)), button, FALSE, TRUE, 5);

  gtk_widget_show_all (about_win);

}

void
goodnight_config_save (GtkWidget * wid, gpointer data)
{
  gchar *configfile, *action;
  ConfigFile *config;

  configfile = g_strconcat (g_get_home_dir (), CONFIGFILE, NULL);

  if ((config = xmms_cfg_open_file (configfile)) == NULL)
    config = xmms_cfg_new ();

  fadespeed = (gint) GTK_ADJUSTMENT (fadespeed_o)->value;
  standby = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (standby_radio));
  shutdown = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (shutdown_radio));
  quit = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (quit_radio));
  hour = (gint) GTK_ADJUSTMENT (hour_w)->value;
  minute = (gint) GTK_ADJUSTMENT (minute_w)->value;
  shutdown_command = g_strdup (gtk_entry_get_text (GTK_ENTRY (shutdown_entry)));
  standby_command = g_strdup (gtk_entry_get_text (GTK_ENTRY (standby_entry)));

  if (quit)
    action = QUIT;
  else if (shutdown)
    action = SHUTDOWN;
  else if (standby)
    action = STANDBY;
  else
    action = STOP;

  if (!shutdown_command)
    shutdown_command = dummy;
  if (!standby_command)
    standby_command = dummy;

  xmms_cfg_write_int (config, "goodnight", "hour", hour);
  xmms_cfg_write_int (config, "goodnight", "minute", minute);
  xmms_cfg_write_int (config, "goodnight", "fadespeed", fadespeed);
  xmms_cfg_write_string (config, "goodnight", "shutdown_command", shutdown_command);
  xmms_cfg_write_string (config, "goodnight", "standby_command", standby_command);
  xmms_cfg_write_string (config, "goodnight", "exit_action", action);

  xmms_cfg_write_file (config, configfile);
  xmms_cfg_free (config);
  g_free (configfile);

  return;
}

void
goodnight_config_ok (GtkWidget * wid, gpointer data)
{
  goodnight_config_save (NULL, NULL);
  gtk_widget_destroy (conf_dialog);
  conf_dialog = NULL;
  return;
}

void
goodnight_config ()
{
  GtkWidget *ok_button, *apply_button, *cancel_button;
  GtkWidget *timebox, *hourfield, *minutefield/*, *fadefield*/;
  GtkWidget *exit_actions, *nothing_radio;
  GtkWidget *timeframe, *actionframe, *textframe/*, *fadeframe*/;
  GtkWidget *bigbox, *buttonbox, *commands_table, *shutd_label, *standby_label;
  GtkWidget *fadescale;

  if ((hour == -1) || (minute == -1))
    {
      read_config (NULL, NULL);
    }

  if (conf_dialog)
    return;

  conf_dialog = gtk_window_new (GTK_WINDOW_DIALOG);

  gtk_window_set_title (GTK_WINDOW (conf_dialog), ("XMMS-Goodnight setup"));
  gtk_window_set_policy (GTK_WINDOW (conf_dialog), FALSE, FALSE, FALSE);
  gtk_window_set_position (GTK_WINDOW (conf_dialog), GTK_WIN_POS_MOUSE);

  gtk_container_set_border_width (GTK_CONTAINER (conf_dialog), 5);

  gtk_signal_connect (GTK_OBJECT (conf_dialog), "destroy", GTK_SIGNAL_FUNC (gtk_widget_destroyed),
		      &conf_dialog);

  bigbox = gtk_vbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER (GTK_WINDOW (conf_dialog)), bigbox);

  timeframe = gtk_frame_new ("Stop playing time:");
  gtk_container_add (GTK_CONTAINER (bigbox), timeframe);

  timebox = gtk_hbox_new (FALSE, 5);
  gtk_container_set_border_width (GTK_CONTAINER (timebox), 5);
  gtk_container_add (GTK_CONTAINER (timeframe), timebox);

  hour_w = gtk_adjustment_new (hour, 0, 23, 1, 1, 1);
  hourfield = gtk_spin_button_new (GTK_ADJUSTMENT (hour_w), 1.0, 0);

  minute_w = gtk_adjustment_new (minute, 0, 59, 1, 1, 1);
  minutefield = gtk_spin_button_new (GTK_ADJUSTMENT (minute_w), 1.0, 0);

  gtk_box_pack_start (GTK_BOX (timebox), gtk_label_new ("Hour: "), FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (timebox), hourfield, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (timebox), gtk_label_new ("Minute: "), FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (timebox), minutefield, FALSE, FALSE, 0);

  fadespeed_o = gtk_adjustment_new (fadespeed, 1, 20, 1.0, 5.0, 1.0);
  fadescale = gtk_spin_button_new (GTK_ADJUSTMENT (fadespeed_o), 1.0, 0);

  gtk_box_pack_start (GTK_BOX (timebox), gtk_label_new ("Fade speed:"), FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (timebox), fadescale, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (timebox), gtk_label_new ("%/sec"), FALSE, FALSE, 0);

  actionframe = gtk_frame_new ("Exit actions:");
  gtk_container_add (GTK_CONTAINER (bigbox), actionframe);

  exit_actions = gtk_vbox_new (FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (exit_actions), 5);

  gtk_container_add (GTK_CONTAINER (actionframe), exit_actions);

  shutdown_radio = gtk_radio_button_new_with_label (NULL, "Shutdown computer");
  gtk_box_pack_start (GTK_BOX (exit_actions), shutdown_radio, TRUE, TRUE, 0);

  standby_radio = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (shutdown_radio), "Standby");
  gtk_box_pack_start (GTK_BOX (exit_actions), standby_radio, TRUE, TRUE, 0);

  quit_radio = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (shutdown_radio), "Exit XMMS");
  gtk_box_pack_start (GTK_BOX (exit_actions), quit_radio, TRUE, TRUE, 0);

  nothing_radio = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (shutdown_radio), "Stop playing");
  gtk_box_pack_start (GTK_BOX (exit_actions), nothing_radio, TRUE, TRUE, 0);

  if ((shutdown))
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shutdown_radio), TRUE);

  if ((standby))
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (standby_radio), TRUE);

  if ((quit))
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (quit_radio), TRUE);

  if ((!shutdown) && (!quit) && (!standby))
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (nothing_radio), TRUE);

  textframe = gtk_frame_new ("Commands:");
  gtk_container_add (GTK_CONTAINER (bigbox), textframe);
  commands_table = gtk_table_new (2, 2, FALSE);
  gtk_container_add (GTK_CONTAINER (textframe), commands_table);

  shutd_label = gtk_label_new ("Shutdown command:");
  standby_label = gtk_label_new ("Standby command:");

  gtk_table_attach_defaults (GTK_TABLE (commands_table), shutd_label, 0, 1, 0, 1);
  gtk_table_attach_defaults (GTK_TABLE (commands_table), standby_label, 0, 1, 1, 2);

  shutdown_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (shutdown_entry), shutdown_command);
  standby_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (standby_entry), standby_command);

  gtk_table_attach_defaults (GTK_TABLE (commands_table), shutdown_entry, 1, 2, 0, 1);
  gtk_table_attach_defaults (GTK_TABLE (commands_table), standby_entry, 1, 2, 1, 2);

/*
   fadeframe=gtk_frame_new("Fade speed (%/sec)");
   gtk_container_add(GTK_CONTAINER(bigbox),fadeframe);

   fadespeed_o=gtk_adjustment_new(fadespeed,1,20,1.0,5.0,1.0);
   fadescale=gtk_spin_button_new(GTK_ADJUSTMENT(fadespeed_o),1.0,0);
   gtk_container_add(GTK_CONTAINER(fadeframe),fadescale);
 */

  buttonbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (buttonbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (buttonbox), 5);
  gtk_box_pack_start (GTK_BOX (bigbox), buttonbox, FALSE, FALSE, 0);

  ok_button = gtk_button_new_with_label ("Ok");
  apply_button = gtk_button_new_with_label ("Apply");
  cancel_button = gtk_button_new_with_label ("Cancel");

  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) conf_dialog);

  gtk_signal_connect_object (GTK_OBJECT (apply_button), "clicked", GTK_SIGNAL_FUNC (goodnight_config_save), NULL);
  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked", GTK_SIGNAL_FUNC (goodnight_config_ok), NULL);

  GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
  GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);
  GTK_WIDGET_SET_FLAGS (apply_button, GTK_CAN_DEFAULT);

  gtk_box_pack_start (GTK_BOX (buttonbox), ok_button, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (buttonbox), cancel_button, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (buttonbox), apply_button, TRUE, TRUE, 0);

  gtk_widget_show_all (conf_dialog);

}
